################################################################################
##
## Filename: nuke_px.py
##
## Copyright (c) 2017 Autodesk, Inc.
## All rights reserved.
##
## Use of this software is subject to the terms of the Autodesk license
## agreement provided at the time of installation or download, or which
## otherwise accompanies this software in either electronic or hard copy form.
################################################################################

import os
import sys
import subprocess
import pybox_v1 as pybox

PAYLOAD_PATH = "/opt/Autodesk/flame_2020/Desktop/MyFLMPresets/payload.json"
NUKE_PATH = "/usr/local/Nuke11.1v4/Nuke11.1" 

def is_basic_ui_created(self):
   elems = self.get_global_elements()
   return "Nuke Composition" in elems

def create_basic_ui(self):
   nuke_composition_browser = pybox.create_file_browser("Nuke Composition", "/", "nk", "/")
   self.add_global_elements(nuke_composition_browser)

   page = pybox.create_page("Main Page", "Browser")
   self.set_ui_pages(page)

class NukePx(pybox.BaseClass):

    def initialize(self):
        if not is_basic_ui_created(self):
            return create_basic_ui(self)

        ui_changes = self.get_ui_changes()
        for el in ui_changes:
            if el["name"] == "Nuke Composition":
               #browser was modified
               nuke_setup = self.get_global_element_value("Nuke Composition")

               self.write_to_disk(PAYLOAD_PATH)

               script = "/opt/Autodesk/shared/presets/pybox/nuke_parse_io.py"
               cmd = NUKE_PATH + " -it " + script + " " + PAYLOAD_PATH + " " + nuke_setup
               os.system(cmd)

               # sync sockets
               p = NukePx(PAYLOAD_PATH)
               self.remove_in_sockets()
               self.remove_out_sockets()

               self.sockets["in"] = p.get_in_sockets()
               self.sockets["out"] = p.get_out_sockets()

               if self.sockets["in"] == []:
                  self.add_in_socket("undefined", "")

               self.ui_changes = []

        self.set_img_format("exr")
        self.set_state_id("setup_ui")
        self.setup_ui()

    def setup_ui(self):
        editor_button = pybox.create_toggle_button("Editor", False, col=0, row=2)
        self.add_global_elements(editor_button)

        self.write_to_disk(PAYLOAD_PATH)
        nuke_setup = self.get_global_element_value("Nuke Composition")
        script = "/opt/Autodesk/shared/presets/pybox/nuke_parse_ui.py"
        cmd = NUKE_PATH + " -it " + script + " " + PAYLOAD_PATH + " " + nuke_setup
        os.system(cmd)

        p = NukePx(PAYLOAD_PATH)
        elems = p.get_render_elements()
        for value in elems.itervalues():
            self.add_render_elements(value)

        self.set_state_id("execute")


    def execute(self):
        nuke_setup = self.get_global_element_value("Nuke Composition")
        for el in self.get_ui_changes():
            if el['name'] == "Editor":
               p = subprocess.Popen([NUKE_PATH, os.path.expanduser(nuke_setup)])
               # remember pids so we can kill everything in the teardown
               if not hasattr(self, "nuke_editors"):
                  self.nuke_editors = []
               self.nuke_editors.append(p.pid)

               # pop back button
               self.set_global_element_value("Editor", False)

            if el["name"] == "Nuke Composition":
               self.dynamic_ui["render_elements"] = {}
               browser = self.get_global_element("Nuke Composition")
               self.dynamic_ui["global_elements"] = {"Nuke Composition": browser}

               self.set_state_id("initialize")
               self.initialize()

        #Render our image
        self.write_to_disk(PAYLOAD_PATH)
        script = "/opt/Autodesk/shared/presets/pybox/nuke_exec.py"
        cmd = (NUKE_PATH + " -ix -F " + str(self.get_frame()) + " " + script + " " + PAYLOAD_PATH + " " +
               nuke_setup)
        os.system(cmd)

    def teardown(self):
        if hasattr(self, "nuke_editors"):
            for i in self.nuke_editors:
               # check if process is nuke (might be something else)
               p = subprocess.Popen(
                     ["ps -o cmd= {}".format(i)], stdout=subprocess.PIPE, shell=True)
               name = str(p.communicate()[0])

               if "Nuke" in name:
                  os.system("kill -9 " + str(i))


################################################################################
##                                 MAIN                                       ##
################################################################################
def _main(argv):

    # Load the json file, make sure you have read access to it
    p = NukePx(argv[0])

    # Call the appropriate function
    p.dispatch()

    # Save file
    p.write_to_disk(argv[0])

if __name__ == '__main__':
    _main(sys.argv[1:])
